' Requires references to:
'       Microsoft.DirectX
'       Microsoft.DirectX.Direct3D
'       System.Windows.Forms
Imports System.Windows.Forms
Imports System.Drawing
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Public Class OctahedronForm
    ' The graphics device.
    Private m_Device As Device = Nothing

    ' Data variables.
    Private Const NUM_TRIANGLES As Integer = 8
    Private Const NUM_POINTS As Integer = 3 * NUM_TRIANGLES

    Private m_VertexBuffer As VertexBuffer = Nothing
    Private m_PresentParams As New PresentParameters()

    ' Initialize Direct3D.
    Public Function InitializeGraphics() As Boolean
        Try
            ' Initialize presentation parameters.
            m_PresentParams.Windowed = True
            m_PresentParams.SwapEffect = SwapEffect.Discard

            ' Make the device.
            m_Device = New Device(0, DeviceType.Hardware, _
                Me, CreateFlags.SoftwareVertexProcessing, _
                m_PresentParams)

            ' Add an event handler for DeviceReset.
            ' This event is called when the device is reset
            ' (you probably guessed that). That happens when
            ' the form is recreated when it is maximized
            ' or restored.
            AddHandler m_Device.DeviceReset, _
                AddressOf Me.OnResetDevice

            ' Call CreateVertexBuffer and OnResetDevice.
            Me.CreateVertexBuffer()
            Me.OnResetDevice(m_Device, Nothing)
            Return True
        Catch e As DirectXException
            Return False
        End Try
    End Function

    ' Create a vertex buffer for the device.
    Public Sub CreateVertexBuffer()
        ' Create a vertex buffer.
        m_VertexBuffer = New VertexBuffer( _
            GetType(CustomVertex.PositionColored), _
            NUM_POINTS, m_Device, 0, CustomVertex.PositionColored.Format, _
            Pool.Default)

        ' Add an event handler for m_VertexBuffer.Created.
        ' This event occurs when the device is recreated
        ' and the vertex buffer is re-initialized.
        AddHandler m_VertexBuffer.Created, _
            AddressOf Me.OnCreateVertexBuffer

        ' Call m_VertexBuffer.Created to initialize 
        ' the vertex data.
        Me.OnCreateVertexBuffer(m_VertexBuffer, Nothing)
    End Sub

    ' Reset the device.
    Public Sub OnResetDevice(ByVal sender As Object, ByVal e As EventArgs)
        ' Turn off D3D lighting because 
        ' we set the vertex colors explicitly.
        Dim dev As Device = CType(sender, Device)
        dev.RenderState.Lighting = False
    End Sub

    ' Define in the vertics.
    Public Sub OnCreateVertexBuffer(ByVal sender As Object, ByVal e As EventArgs)
        Dim vb As VertexBuffer = CType(sender, VertexBuffer)

        ' Lock the vertex buffer. 
        ' Lock returns an array of positionColored objects.
        Dim vertices As CustomVertex.PositionColored() = _
            CType(vb.Lock(0, 0), _
                CustomVertex.PositionColored())
        Dim start_index As Integer = 0

        Const WID As Single = 2
        MakeTriangle(vertices, start_index, Color.Red, _
            0, WID, 0, _
            0, 0, WID, _
            WID, 0, 0)
        MakeTriangle(vertices, start_index, Color.Yellow, _
            0, WID, 0, _
            WID, 0, 0, _
            0, 0, -WID)
        MakeTriangle(vertices, start_index, Color.Red, _
            0, WID, 0, _
            0, 0, -WID, _
            -WID, 0, 0)
        MakeTriangle(vertices, start_index, Color.Yellow, _
            0, WID, 0, _
            -WID, 0, 0, _
            0, 0, WID)
        MakeTriangle(vertices, start_index, Color.Blue, _
            0, -WID, 0, _
            WID, 0, 0, _
            0, 0, WID)
        MakeTriangle(vertices, start_index, Color.Green, _
            0, -WID, 0, _
            0, 0, -WID, _
            WID, 0, 0)
        MakeTriangle(vertices, start_index, Color.Blue, _
            0, -WID, 0, _
            -WID, 0, 0, _
            0, 0, -WID)
        MakeTriangle(vertices, start_index, Color.Green, _
            0, -WID, 0, _
            0, 0, WID, _
            -WID, 0, 0)

        vb.Unlock()
    End Sub

    Private Sub MakeTriangle( _
        ByVal vertices As CustomVertex.PositionColored(), _
        ByRef start_index As Integer, ByVal clr As Color, _
        ByVal x1 As Single, ByVal y1 As Single, ByVal z1 As Single, _
        ByVal x2 As Single, ByVal y2 As Single, ByVal z2 As Single, _
        ByVal x3 As Single, ByVal y3 As Single, ByVal z3 As Single)

        With vertices(start_index)
            .X = x1
            .Y = y1
            .Z = z1
            .Color = clr.ToArgb()
        End With
        start_index += 1

        With vertices(start_index)
            .X = x2
            .Y = y2
            .Z = z2
            .Color = clr.ToArgb()
        End With
        start_index += 1

        With vertices(start_index)
            .X = x3
            .Y = y3
            .Z = z3
            .Color = clr.ToArgb()
        End With
        start_index += 1
    End Sub

    ' Render the scene.
    Public Sub Render()
        ' Do nothing if the device has not been created yet.
        If m_Device Is Nothing Then Return

        ' Clear the back buffer.
        m_Device.Clear(ClearFlags.Target, Color.White, 1, 0)

        ' Begin the scene.
        m_Device.BeginScene()

        ' Setup the world, view, and projection matrices.
        SetupMatrices()

        ' Set the device's data stream source 
        ' (the vertex buffer).
        m_Device.SetStreamSource(0, m_VertexBuffer, 0)

        ' Tell the device the format of the vertices.
        m_Device.VertexFormat = CustomVertex.PositionColored.Format

        ' Draw the primitives in the data stream.
        m_Device.DrawPrimitives(PrimitiveType.TriangleList, 0, NUM_TRIANGLES)

        ' End the scene.
        m_Device.EndScene()

        ' Display the result.
        m_Device.Present()
    End Sub

    ' Setup the world, view, and projection matrices.
    Private Sub SetupMatrices()
        ' World Matrix:
        ' Rotate the object around the Y axis by
        ' 2 * Pi radians per 2000 ticks (2 seconds).
        Const TICKS_PER_REV As Integer = 2000
        Dim ms_rotated As Integer = Environment.TickCount Mod TICKS_PER_REV
        Dim angle As Double = ms_rotated * (2 * Math.PI) / TICKS_PER_REV
        m_Device.Transform.World = Matrix.RotationY(CSng(angle))

        ' View Matrix:
        ' This is defined by giving:
        '       An eye point            (0, 3, -10)
        '       A point to look at      (0, 0, 0)
        '       An "up" direction       <0, 1, 0>
        m_Device.Transform.View = Matrix.LookAtLH( _
            New Vector3(0, 3, -10), _
            New Vector3(0, 0, 0), _
            New Vector3(0, 1, 0))

        ' Projection Matrix:
        ' Perspective transformation defined by:
        '       Field of view           Pi / 4
        '       Aspect ratio            1
        '       Near clipping plane     Z = 1
        '       Far clipping plane      Z = 100
        m_Device.Transform.Projection = _
            Matrix.PerspectiveFovLH(Math.PI / 4, 1, 1, 100)
    End Sub
End Class